import aiohttp
import requests

headers = {
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; WOW64; Trident/7.0; rv:11.0) like Gecko"
}


def fetch(vURL: str) -> str:
    with requests.get(vURL, headers=headers) as response:
        RAW = response.text
    return RAW


async def async_fetch(vURL: str, local_addr: str = None) -> str:
    connector = aiohttp.TCPConnector(local_addr=(local_addr, 0)) if local_addr else None
    async with aiohttp.ClientSession(connector=connector, headers=headers) as session:
        async with session.get(vURL) as response:
            RAW = await response.text()
    return RAW



# -*- coding: utf-8 -*-
"""
Youtube Related Video Fetcher
"""

__title__ = "youtube_related"
__author__ = "kijk2869"
__lisence__ = "MIT"
__version__ = "0.0.2"

from collections import namedtuple

from .client import async_get, get, preventDuplication
from .fetcher import async_fetch, fetch
from .parser import loadInitialData, parse

VersionInfo = namedtuple("VersionInfo", "major minor micro releaselevel serial")

version_info = VersionInfo(major=0, minor=0, micro=2, releaselevel="final", serial=0)



import json
import re

DATA_JSON = re.compile(r'window\["ytInitialData"\]\s\=\s(\{.*\})')


def __loadInitialData(RAW: str) -> dict:
    Search = DATA_JSON.search(RAW)

    if not Search:
        raise ValueError

    RAW_DATA = Search.group(1)
    Data = json.loads(RAW_DATA)

    return Data


def parse(RAW: str) -> list:
    Data = __loadInitialData(RAW)

    Overlay = Data["playerOverlays"]["playerOverlayRenderer"]
    watchNextEndScreenRenderer = Overlay["endScreen"]["watchNextEndScreenRenderer"]
    Result = [
        {
            "id": Item["videoId"],
            "title": Item["title"]["simpleText"],
            "duration": Item["lengthInSeconds"] if "lengthInSeconds" in Item else None,
        }
        for Item in [
            result["endScreenVideoRenderer"]
            for result in watchNextEndScreenRenderer["results"]
            if "endScreenVideoRenderer" in result
        ]
    ]

    return Result



from collections import deque

from .fetcher import async_fetch, fetch
from .parser import parse


def get(vURL: str) -> list:
    RAW = fetch(vURL)
    Data = parse(RAW)

    return Data


async def async_get(vURL: str, local_addr: str = None) -> list:
    RAW = await async_fetch(vURL, local_addr)
    Data = parse(RAW)

    return Data


class preventDuplication:
    def __init__(self):
        self._LastRelated = deque(maxlen=10)

    def get(self, vURL: str) -> dict:
        Data = get(vURL)

        for Item in Data:
            if not Item["id"] in self._LastRelated:
                self._LastRelated.append(Item["id"])
                return Item

        self._LastRelated.clear()
        return Data[0]

    async def async_get(self, vURL: str, local_addr: str = None) -> dict:
        Data = await async_get(vURL, local_addr)

        for Item in Data:
            if not Item["id"] in self._LastRelated:
                self._LastRelated.append(Item["id"])
                return Item

        self._LastRelated.clear()
        return Data[0]
